/****************************************************************************
 * Filename : SharewareService.asmx.cs
 * Project  : Shareware Starter Kit: SharewareService
 * 
 * File Description 
 * 
 *	Implements the Web Server Methods and the database read/write methods
 * 
 * Implemented Namespaces
 * 
 *		SSK.Server
 * 
 *
 ****************************************************************************/

using System;
using System.Web;
using System.Collections;
using System.Configuration;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Diagnostics;
using System.Xml;
using System.Xml.Serialization;
using System.IO;


using System.Text;

using SSK.Server;
using SSK.Client.License;


namespace SSK.Server.Webservice
{

    /// <summary>
    /// Defines and implements the Web Methods for the Shareware Starter Kit
    ///	Server.
    /// </summary>
    [WebService(Namespace = "http://tempuri.org/")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    public class SharewareService : System.Web.Services.WebService
    {

        private Logger logger = new Logger(ConfigurationManager.AppSettings["loggingPath"], "SharewareServiceLog.txt");
        private SharewareServiceImpl        serviceImpl = new SharewareServiceImpl();

        /// <summary>
        /// RegisterUser allows a Shareware Starter Kit
        /// application (client) to register a user for software 
        /// and information updates with the shareware developer.
        /// This information is also referenced on the server 
        /// for other actions done by the client user.  This method 
        /// validates input for SQL processing and uses the database
        /// helper class to access/read the database.  If the required 
        /// parameters are not entered a ArgumentNullException is
        /// thrown.
        /// </summary>
        /// <param name="lastname">Required: Last name </param>
        /// <param name="firstname">Required: First name </param>
        /// <param name="address">Address</param>
        /// <param name="city">City</param>
        /// <param name="state">State</param>
        /// <param name="zip">Zip code (##### or #####-####)</param>
        /// <param name="phone">Phone ((###) ###-####)</param>
        /// <param name="email">Required: Email (name@server.com)</param>
        /// <returns></returns>
        [WebMethod]
        public Boolean RegisterUser
        (
            string lastname,		// required
            string firstname,							// required
            string address,
            string city,
            string state,
            string zip,
            string phone,
            string email,							    // required 						
            string devToken,
            string productName,
            string uniqueMachineID,
            string licenseLevel
        )                              
        {
            Logger.Log("WebService : RegisterUser : Beginning ");

            Boolean retVal = serviceImpl.RegisterUser( lastname, firstname, address, city, state, zip, phone, email, 
                                        devToken, productName, uniqueMachineID, licenseLevel );
                                        
            Logger.Log("WebService : RegisterUser : End ");
            return retVal;
        }




        /// <summary>
        /// GetLicense is the WebMethod that implements the Buy Now experience on
        /// the Shareware Starter Kit Server.  The client application calls this
        /// method to request a LicenseResponse object.
        /// </summary>
        /// 
        /// <param name="licenseRequest">
        /// LicenseRequest object which contains
        /// Product Name, Client Identify, and License Level
        /// </param>
        /// 
        /// <param name="invoice"> 
        /// Invoice is an identifier created on the client
        /// for future referencing.
        /// </param>
        /// 
        /// <returns>
        /// A LicenseResponse object if the Invoice ID is able to give 
        /// licenses or NULL if not.
        /// </returns>
        [WebMethod]
        public string GetLicense
        (
            string product,
            string clientIdentity,
            string licenseLevel,
            string invoice,
            string devToken,
            string clientSerialId
        )
        {
            Logger.Log("WebService : GetLicense : Beginning ");

            string retVal = serviceImpl.GetLicense( product, clientIdentity, licenseLevel, 
                                                    invoice, devToken, clientSerialId);

            Logger.Log("WebService : GetLicense : End ");
            return retVal;
        }

        /// <summary>
        /// SubmitIssue allows a client application to send an issue to the
        /// Shareware Server.
        /// 
        /// This method validates input for SQL processing.
        /// Uses the database helper class.
        /// Throws ArgumentNullException for all null or empty paramters.
        /// </summary>
        /// <param name="productName">Name of the product</param>
        /// <param name="licenseLevel">License Level</param>
        /// <param name="emailContact">Email contact (name@server.com)</param>
        /// <param name="firstName">First Name</param>
        /// <param name="lastName">Last Name</param>
        /// <param name="description">Description of the feedback the client
        /// would like to give the server</param>
        [WebMethod]
        public Boolean SubmitIssue
        (
            string          productName, 
            string          licenseLevel, 
            string          emailContact, 
            string          firstName,
            string          lastName, 
            string          description, 
            string          devID, 
            string          uniqueMachineID
        )
        {
            Logger.Log("WebService : SubmitIssue : Beginning ");

            Boolean retVal = serviceImpl.SubmitIssue( productName, licenseLevel, emailContact, firstName,
                                                      lastName, description, devID, uniqueMachineID );

            Logger.Log("WebService : SubmitIssue : End ");
            return retVal;
        }

        /// <summary>
        /// SubmitException is used by the shareware client to submit
        /// an exception that happened when the shareware user was using
        /// the shareware application.
        /// </summary>
        /// <param name="message">Exception Message</param>
        /// <param name="source">Source filename which threw the exception</param>
        /// <param name="stackTrace">Stack trace at exception</param>
        /// <param name="methodName">Method name which threw the exception</param>
        /// <param name="productName">The name of the product throwing the exception</param>
        [WebMethod]
        public Boolean SubmitException
        (
            string          message,
            string          source,
            string          stackTrace,
            string          methodName,
            string          productName,
            string          devToken,
            string          uniqueMachineID,
            string          licenseLevel
        )
        {
            Logger.Log("WebService : SubmitException : Beginning ");

            Boolean retVal = serviceImpl.SubmitException( message, source, stackTrace, methodName,
                                               productName, devToken, uniqueMachineID, licenseLevel );

            Logger.Log("WebService : SubmitException : End ");
            return retVal;
        }

        /// <summary>
        /// GetClientAppConfig is used to return the application configuration setting back to the client
        /// for storage in the local application configuration
        /// </summary>
        [WebMethod]
        public ClientAppConfig GetClientAppConfig
        (
            string          productName,
            string          devToken,
            string          uniqueMachineID,
            DateTime        startDateTime
        )
        {
            Logger.Log("WebService : GetClientAppConfig : Beginning ");

            ClientAppConfig retVal = serviceImpl.GetClientAppConfig(productName, devToken, uniqueMachineID, startDateTime, this.Context.Request.Url.ToString());

            Logger.Log("WebService : GetClientAppConfig : End ");
            return retVal;
        }

    }


}

