using System;
using System.Collections;
using System.ComponentModel;
using System.Configuration;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Diagnostics;
using System.Net;
using System.IO;
using PayPal.Web.Controls;
using SSK.Server;

namespace PayPal.IPN
{
    /// <summary>
    /// Handles the IPN post back message from PayPal, converts the IPN message into a 
    /// corresponding <see cref="IpnTransaction"/> and then calls the <see cref="IpnService"/>
    /// web service to process this IPN message.
    /// </summary>
    /// <remarks>
    /// This page is used to handle the current PayPal IPN system. Future releases may
    /// not require this page.
    /// </remarks>
  
    public partial class IpnHandler : System.Web.UI.Page
    {
        private Logger IpnHandlerServerlogger = new Logger(ConfigurationSettings.AppSettings["loggingPath"], "SharewareServiceLog.txt");

        private void Page_Load(object sender, System.EventArgs e)
        {
            Logger.Log("IpnHandler : Beginning");

            string strFormValues;

            strFormValues = Request.Form.ToString();

            string strNewValue;
            string strResponse;


            // Create the request back
            HttpWebRequest req = (HttpWebRequest)WebRequest.Create("http://www.paypal.com/cgi-bin/webscr");

            // Set values for the request back
            req.Method = "POST";
            req.ContentType = "application/x-www-form-urlencoded";
            strNewValue = strFormValues + "&cmd=_notify-validate";
            req.ContentLength = strNewValue.Length;

            // Write the request back IPN strings
            StreamWriter stOut = new StreamWriter(req.GetRequestStream(), System.Text.Encoding.ASCII);
            stOut.Write(strNewValue);
            stOut.Close();

            // Do the request to PayPal and get the response
            StreamReader stIn = new StreamReader(req.GetResponse().GetResponseStream());
            strResponse = stIn.ReadToEnd();
            stIn.Close();

            // Confirm whether the IPN was VERIFIED or INVALID. If INVALID, just ignore the IPN
            if (strResponse == "VERIFIED")
            {
                Logger.Log("IpnHandler : Verified");

                // Create the IpnTransaction
                IpnTransaction ipn = CreateIpnTransaction();

                // Make the request to the web service
                IpnService service = new IpnService();
                Logger.Log("IpnHandler : service.ProcessTransaction : Beginning");
                service.ProcessTransaction(ipn);
                Logger.Log("IpnHandler : service.ProcessTransaction : Completed");
            }
            else
            {
                Logger.Log("IpnHandler : NOT Verified");
            }
        }


        #region Helper Methods used to create an IpnTransaction object

        /// <summary>
        /// Creates an <see cref="IpnTransaction"/> based on the current instance of the IPN message.
        /// </summary>
        /// <remarks>
        /// This will use the current <see cref="HttpRequest.Form"/> values.
        /// </remarks>
        /// <returns>An instance of <see cref="IpnTransaction"/> that represents the current IPN.</returns>
        private IpnTransaction CreateIpnTransaction()
        {
            IpnTransaction ipn = new IpnTransaction();

            // Do some preprocessing first
            if (Request.Form["txn_type"] == "cart")
            {
                // Figure out how many items in the cart
                int cartCount = Int32.Parse(Request.Form["num_cart_items"]);

                for (int i = 0; i < cartCount; i++)
                {
                    ipn.Items.Add(new CartItem());
                }
            }

            // Loop through all the keys in the Form
            Debug.Assert(Request.Form != null, "Form was null!");
            Debug.Assert(Request.Form.Keys != null, "Form Keys was null!");
            foreach (string key in Request.Form.Keys)
            {
                try
                {
                    SetIpnTransactionProperties(key, Request.Form[key], ipn);
                }
                catch (Exception)
                {
                    // An error in parsing the key/value will be skipped
                    continue;
                }
            }

            ipn.RawIPN = Request.Form.ToString();
            return ipn;
        }

        /// <summary>
        /// Converts a date and time format returned by the IPN message into a 
        /// <see cref="DateTime"/> value.
        /// <remarks>
        /// Abides by the IPN date time format as specified in the IPN specifications
        /// dated 9/19/2003. Must always be in the following sample format:<br/>
        /// <code>18:30:30 Jan 01, 2000 PDT</code>
        /// </remarks>
        /// </summary>
        /// <param name="strIpnDate">The original IPN date and time.</param>
        /// <returns>A <see cref="DateTime"/> of the IPN date and time.</returns>
        private static DateTime ConvertIpnDate(string strIpnDate)
        {
            // Get rid of the TimeZone at the end of the IPN Date Time
            string date = strIpnDate.Substring(0, strIpnDate.Length - 3).Trim();

            int nHour, nMinute, nSecond;
            int nMonth, nDay, nYear;

            try
            {
                // Hard coded beginning/length values

                nHour = Int32.Parse(date.Substring(0, 2));
                nMinute = Int32.Parse(date.Substring(3, 2));
                nSecond = Int32.Parse(date.Substring(6, 2));

                nMonth = DateTime.ParseExact(date.Substring(9, 3), "MMM", null).Month;
                nDay = Int32.Parse(date.Substring(13, 2));
                nYear = Int32.Parse(date.Substring(17, 4));

                return new DateTime(nYear, nMonth, nDay, nHour, nMinute, nSecond);
            }
            catch (FormatException e)
            {
                // Invalid date time format, throw the error
                throw e;
            }
        }


        /// <summary>
        /// Helper function used to set the property value for an <see cref="IpnTransaction"/> object
        /// for the given key.
        /// </summary>
        /// <param name="key">The key used to find the property.</param>
        /// <param name="val">The value to set to the property.</param>
        /// <param name="ipn">The <see cref="IpnTransaction"/> to set to.</param>
        private void SetIpnTransactionProperties(string key, string val, IpnTransaction ipn)
        {
            Debug.Assert(key != null, "Cannot find property with key == null!");
            Debug.Assert(val != null, "Cannot set property with val == null!");
            Debug.Assert(ipn != null, "Cannot set property with ipn == null!");


            // Set Date/Time Properties
            if (SetIpnTransactionProperties_Date(key, val, ipn))
            {
                return;
            }

            // Set Buyer Information Properties
            else if (SetIpnTransactionProperties_Customer(key, val, ipn))
            {
                return;
            }

            // Set Product Information Properties
            else if (SetIpnTransactionProperties_Product(key, val, ipn))
            {
                return;
            }

            // Set Subscriptions Information Properties
            else if (SetIpnTransactionProperties_SubscriptionItem(key, val, ipn))
            {
                return;
            }

            // Set Generically
            else
            {
                ipn.SetProperty(key, val);
            }

        }

        #region SetIpnTransactionProperties helper methods

        /// <summary>
        /// Used to set IPN values related to a subscription.
        /// </summary>
        /// <remarks>
        /// This method should not be used directly. Instead use
        /// <see cref="SetIpnTransactionProperties"/>.
        /// </remarks>
        /// <param name="key">The key used to find the property.</param>
        /// <param name="val">The value to set to the property.</param>
        /// <param name="ipn">The <see cref="IpnTransaction"/> to set to.</param>
        /// <returns>TRUE if a property was set, otherwise returns FALSE.</returns>
        private bool SetIpnTransactionProperties_SubscriptionItem(string key, string val, IpnTransaction ipn)
        {
            // Only want to set these values
            int nSubItemIndex;
            switch (key)
            {
                case "amount1":
                case "amount2":
                case "amount3":
                    nSubItemIndex = Int32.Parse(key.Substring(6)) - 1;
                    ipn.SubscriptionItems[nSubItemIndex].Amount = Convert.ToSingle(val);
                    break;

                case "period1":
                case "period2":
                case "period3":
                    nSubItemIndex = Int32.Parse(key.Substring(6)) - 1;

                    // Parse the period IPN variable into it's parts
                    string strPeriod, strTimeUnit;
                    strPeriod = val.Substring(0, val.IndexOf(' '));
                    strTimeUnit = val.Substring(val.IndexOf(' ') + 1);

                    // Set corresponding SubscriptionItems properties
                    ipn.SubscriptionItems[nSubItemIndex].Period = Convert.ToInt32(strPeriod);
                    switch (strTimeUnit.ToUpper())
                    {
                        case "D":
                            ipn.SubscriptionItems[nSubItemIndex].TimeUnits = SubscriptionPeriodUnits.Days;
                            break;

                        case "W":
                            ipn.SubscriptionItems[nSubItemIndex].TimeUnits = SubscriptionPeriodUnits.Weeks;
                            break;

                        case "M":
                            ipn.SubscriptionItems[nSubItemIndex].TimeUnits = SubscriptionPeriodUnits.Months;
                            break;

                        case "Y":
                            ipn.SubscriptionItems[nSubItemIndex].TimeUnits = SubscriptionPeriodUnits.Years;
                            break;
                    }
                    break;

                default:
                    return false;
            }

            return true;
        }

        /// <summary>
        /// Used to set IPN values related to a customer.
        /// </summary>
        /// <remarks>
        /// This method should not be used directly. Instead use
        /// <see cref="SetIpnTransactionProperties"/>.
        /// </remarks>
        /// <param name="key">The key used to find the property.</param>
        /// <param name="val">The value to set to the property.</param>
        /// <param name="ipn">The <see cref="IpnTransaction"/> to set to.</param>
        /// <returns>TRUE if a property was set, otherwise returns FALSE.</returns>
        private bool SetIpnTransactionProperties_Customer(string key, string val, IpnTransaction ipn)
        {
            // Only want to set these values

            switch (key)
            {
                case "first_name":
                    ipn.CustomerInfo.FirstName = val;
                    break;

                case "last_name":
                    ipn.CustomerInfo.LastName = val;
                    break;

                case "payer_business_name":
                    ipn.CustomerInfo.BusinessName = val;
                    break;

                case "address_name":
                    ipn.CustomerInfo.AddressName = val;
                    break;

                case "address_street":
                    ipn.CustomerInfo.Address1 = val;
                    break;

                case "address_city":
                    ipn.CustomerInfo.City = val;
                    break;

                case "address_state":
                    ipn.CustomerInfo.State = val;
                    break;

                case "address_zip":
                    ipn.CustomerInfo.Zip = val;
                    break;

                case "address_country":
                    ipn.CustomerInfo.Country = val;
                    break;

                case "address_status":
                    ipn.SetProperty("address_status", val);
                    break;

                case "payer_email":
                    ipn.CustomerInfo.Email = val;
                    break;

                case "payer_id":
                    ipn.CustomerInfo.PayerID = val;
                    break;

                case "payer_status":
                    ipn.SetProperty("payer_status", val);
                    break;

                default:
                    return false;
            }

            return true;
        }

        /// <summary>
        /// Used to set IPN values related to a product.
        /// </summary>
        /// <remarks>
        /// This method should not be used directly. Instead use
        /// <see cref="SetIpnTransactionProperties"/>.
        /// </remarks>
        /// <param name="key">The key used to find the property.</param>
        /// <param name="val">The value to set to the property.</param>
        /// <param name="ipn">The <see cref="IpnTransaction"/> to set to.</param>
        /// <returns>TRUE if a property was set, otherwise returns FALSE.</returns>
        private bool SetIpnTransactionProperties_Product(string key, string val, IpnTransaction ipn)
        {
            if (key.StartsWith("item_name"))
            {
                int itemIndex;

                try
                {
                    itemIndex = Int32.Parse(key.Substring(9));	// 9 is itemIndex after "item_name"
                }
                catch (Exception)
                {
                    // If no itemIndex, then just single item
                    itemIndex = 1;
                }

                if (ipn.Items.Count < itemIndex)
                {
                    for (int i = ipn.Items.Count; i < itemIndex; i++)
                    {
                        ipn.Items.Add(new CartItem());
                    }
                }

                ShoppingItem item = ipn.Items[itemIndex - 1];
                item.ItemName = val;
            }
            else if (key.StartsWith("item_number"))
            {
                int itemIndex;

                try
                {
                    itemIndex = Int32.Parse(key.Substring(11));	// 11 is itemIndex after "item_number"
                }
                catch (Exception)
                {
                    // If no itemIndex, then just single item
                    itemIndex = 1;
                }

                if (ipn.Items.Count < itemIndex)
                {
                    for (int i = ipn.Items.Count; i < itemIndex; i++)
                    {
                        ipn.Items.Add(new CartItem());
                    }
                }

                ShoppingItem item = ipn.Items[itemIndex - 1];
                item.ItemNumber = val;
            }
            else if (key.StartsWith("quantity"))
            {
                int itemIndex;

                try
                {
                    itemIndex = Int32.Parse(key.Substring(8));	// 8 is itemIndex after "quantity"
                }
                catch (Exception)
                {
                    // If no itemIndex, then just single item
                    itemIndex = 1;
                }

                if (ipn.Items.Count < itemIndex)
                {
                    for (int i = ipn.Items.Count; i < itemIndex; i++)
                    {
                        ipn.Items.Add(new CartItem());
                    }
                }

                CartItem item = (CartItem)ipn.Items[itemIndex - 1];
                item.Quantity = Int32.Parse(val);
            }
            else
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// Used to set IPN values with date and time information.
        /// </summary>
        /// <remarks>
        /// This method should not be used directly. Instead use
        /// <see cref="SetIpnTransactionProperties"/>.
        /// </remarks>
        /// <param name="key">The key used to find the property.</param>
        /// <param name="val">The value to set to the property.</param>
        /// <param name="ipn">The <see cref="IpnTransaction"/> to set to.</param>
        /// <returns>TRUE if a property was set, otherwise returns FALSE.</returns>
        private bool SetIpnTransactionProperties_Date(string key, string val, IpnTransaction ipn)
        {
            if (key == "payment_date")
            {
                ipn.PaymentDate = ConvertIpnDate(val);
            }
            else if (key == "auction_closing_date")
            {
                ipn.PaymentDate = ConvertIpnDate(val);
            }
            else
            {
                return false;
            }

            return true;
        }

        #endregion

        #endregion

        #region Web Form Designer generated code
        override protected void OnInit(EventArgs e)
        {
            //
            // CODEGEN: This call is required by the ASP.NET Web Form Designer.
            //
            InitializeComponent();
            base.OnInit(e);
        }

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {

        }
        #endregion
    }
}
